require 'mathn'
require 'active_support/number_helper'

module Planik
  module Lohnausweis

    class Formatter
      attr_reader :align_right

      def initialize align_right = false
        @align_right = align_right
      end

      def summe(arbeitsliste)
        nil
      end
    end

    class DateFormatter < Formatter
      def format datum
        I18n.l(datum, format: :mit_wochentag, locale: :de)
      end
    end

    class TimeFormatter < Formatter
      def format time
        time.nil? ? nil : time.strftime("%H:%M")
      end
    end

    class TimeEndFormatter < Formatter
      def format time
        if time.nil?
          return nil
        end
        s = time.strftime("%H:%M")
        s == "00:00" ? "24:00" : s
      end
    end

    class TimeDeziFormatter < Formatter
      def initialize
        super(true)
      end

      def format time
        #time = nil if time == 0
        time ? super('%.2f', time) : nil
      end


    end

    class NullFormatter < Formatter
      def format text
        text
      end
    end

    #
    # Eine Spalte in der Arbeitliste.
    #
    class Arbeitslistespalte
      attr_reader :key, :text, :relative_breite, :formatter

      def initialize(key, text, relative_breite, formatter)
        @key, @text, @relative_breite, @formatter = key, text, relative_breite, formatter
      end

      def format arbeits_item
        formatter.format(arbeits_item.get(key))
      end

      def summe(arbeitsliste)
        if is_summen_spalte?
          sum = 0
          arbeitsliste.each { |x|
            sum += x.get(key) if x.get(key)
          }
          sum = formatter.format(sum)
        else
          nil
        end
      end

      def is_summen_spalte?
        formatter.is_a? TimeDeziFormatter
      end


    end

    class Arbeitslistespalten
      SPALTEN = {
          datum: Arbeitslistespalte.new(:datum, "Datum", 4, DateFormatter.new),
          name: Arbeitslistespalte.new(:name, "Dienst/Arbeit", 4, NullFormatter.new),
          typ: Arbeitslistespalte.new(:typ, "Typ", 4, NullFormatter.new),
          start: Arbeitslistespalte.new(:start, "Start", 2, TimeFormatter.new),
          ende: Arbeitslistespalte.new(:ende, "Ende", 2, TimeEndFormatter.new),
          arbeitszeit: Arbeitslistespalte.new(:arbeitszeit, "Arbeitszeit", 3, TimeDeziFormatter.new),
          zulage_nacht: Arbeitslistespalte.new(:zulage_nacht, "Zulage Nacht", 4, TimeDeziFormatter.new),
          zulage_wochenende: Arbeitslistespalte.new(:zulage_wochenende, "Zulage Wochenende", 4, TimeDeziFormatter.new)
      }

      def self.find(key)
        x = SPALTEN[key]
        raise "Kann Arbeitslistespalte für key '#{key}' nicht finden" unless x
        x
      end
    end

  end


end