module Planik
  module Lohnausweis

    class Daten

      class Block
        attr_reader :titel_leiste
        attr_reader :eintraege

        def initialize eintraege, titel_leiste = nil
          @eintraege, @titel_leiste = eintraege, titel_leiste
        end

        def daten
          d = []
          d << titel_leiste unless titel_leiste.nil?

          eintraege.each do |e|
            x = []
            x << e.name
            x << e.menge_to_s
            x << e.ansatz_to_s
            x << e.betrag_gerundet_to_s
            d << x
          end
          d
        end

      end


      def initialize(rohdaten)
        @rohdaten = rohdaten
        create_zulagen_block
        create_arbeitszeit_block
        create_arbeitsliste
      end

      # Neu
      # Aenderung für MEDPHONE März 2015: Nur Arbeitsliste, Arbeitszeit und Zulagen
      def create_zulagen_block

        zulagen = []
        if @rohdaten.geldbetraege_zulagen_anzeigen
          zulagen << Zulage.new("Zulage Nacht (#{monat} #{jahr})", @rohdaten.zulage_nacht.menge, @rohdaten.zulage_nacht.ansatz)
          zulagen << Zulage.new("Zulage Wochenende (#{monat} #{jahr})", @rohdaten.zulage_wochenende.menge, @rohdaten.zulage_wochenende.ansatz)
          summe = Rechnung.new("Summe", zulagen, [])

          zulagen = zulagen + [summe]

          @zulagen_block = Block.new(zulagen, ["Zulagen", "Menge", "Ansatz / %", "Betrag"])
        else
          zulagen << Fakt.new("Zulage Nacht (#{monat} #{jahr})", nil, nil, @rohdaten.zulage_nacht.menge, nil)
          zulagen << Fakt.new("Zulage Wochenende (#{monat} #{jahr})", nil, nil, @rohdaten.zulage_wochenende.menge, nil)
          summe = Rechnung.new("Summe", zulagen, [], '')
          zulagen = zulagen + [summe]

          @zulagen_block = Block.new(zulagen, ["Zulagen", nil, nil, "Stunden"])
        end

      end


      def create_arbeitszeit_block
        eintraege = []
        titel = ["Arbeitszeit #{monat} #{jahr}", nil, nil, "Stunden"]

        eintraege << Fakt.new("Stundensaldo Vormonat", nil, nil, @rohdaten.arbeitszeit.gleitzeit_vor, nil)
        eintraege << Fakt.new("Sollstunden von #{start_datum} bis #{end_datum}", nil, nil, @rohdaten.arbeitszeit.soll_stunden, nil)
        eintraege << Fakt.new("Geleistete Stunden von #{start_datum} bis #{end_datum}", nil, nil, @rohdaten.arbeitszeit.ist_stunden, nil)
        eintraege << Rechnung.new("Saldo per #{end_datum}", [eintraege[0], eintraege[2]], [eintraege[1]], nil)

        @arbeitszeit_block = Block.new(eintraege, titel)
      end

      # erstellt die Arbeitliste {:spalten, :eintraege}
      # [titel, arbeitsitems, Total]
      def create_arbeitsliste

        eintraege = []
        titel = @rohdaten.arbeitsliste_spalten.map{|x| x.text}#["Datum", "Dienst/Arbeit", "Typ", "Start", "Ende", "Arbeitszeit"]
        eintraege << titel

        @rohdaten.arbeitsliste.each do |a|
          d = []
          @rohdaten.arbeitsliste_spalten.each do |spalte|
            d << spalte.format(a)
          end
          eintraege << d
        end

        last = Array.new#(@rohdaten.arbeitsliste_spalten.length)

        @rohdaten.arbeitsliste_spalten.each do |a|
          last << a.summe(@rohdaten.arbeitsliste)
        end
        last[0] = "Arbeitszeit total"

        eintraege << last

        al = {spalten: @rohdaten.arbeitsliste_spalten, eintraege: eintraege}
        @arbeitsliste = al
      end


      def adresse
        @rohdaten.adresse
      end

      # Neu
      # Aenderung für MEDPHONE März 2015: Nur Arbeitsliste, Arbeitszeit und Zulagen
      def zulagen_daten
        @zulagen_block.daten
      end

      def arbeitszeit_daten
        @arbeitszeit_block.daten
      end

      def arbeitsliste
        @arbeitsliste
      end

      def start_datum
        @rohdaten.start_datum.strftime("%d.%m.%Y")
      end

      def end_datum
        @rohdaten.end_datum.strftime("%d.%m.%Y")
      end

      def monat
        I18n.localize @rohdaten.start_datum, :format => "%B", :locale => :de
      end

      def jahr
        @rohdaten.start_datum.strftime("%Y")
      end

    end
  end
end

