require 'mathn'
require 'active_support/number_helper'

module Planik
  module Lohnausweis

    # Abstrakte Klasse für alle Einträge in den Tabellen des Lohnausweises
    # Ein Eintrag hat vier mögliche Spalten: Name, Menge, Ansatz und Betrag, von denen aber der einte oder andere nil sein kann.
    class Eintrag
      attr_reader :name, :menge, :ansatz, :einheit

      def initialize(name, menge, ansatz, einheit = "Fr")
        @name, @menge, @ansatz, @einheit= name, menge, ansatz, einheit
      end

      def betrag
        raise "implement in subclass"
      end

      # Die Menge, gerundet auf zwei Kommastellen genau
      def menge_to_s
        return menge if menge.instance_of? String
        menge.nil? ? nil :format('%.2f', menge)
      end

      # Default Wert ist nil
      def ansatz_to_s
        nil
      end

      def betrag_gerundet_to_s
        value_gerundet_to_s betrag
      end

      private

      def value_gerundet_to_s value
        value = betrag_gerundet(value) if einheit == "Fr"
        x = format('%.2f', value)
        x = ActiveSupport::NumberHelper.number_to_delimited(x, delimiter: "'")
        # x = ActiveSupport::NumberHelper.number_to_rounded(value, precision: 2)
        # x = ActiveSupport::NumberHelper.number_to_delimited(x, delimiter: "'")
        x = "#{einheit} #{x}" if einheit
        x
      end


      def betrag_gerundet(_betrag)
        (_betrag * 2).round(1) / 2.0
      end

    end


    class Fakt < Eintrag
      attr_reader :betrag

      def initialize(name, menge, ansatz, betrag, einheit = "Fr")
        super(name, menge, ansatz, einheit)
        @betrag = betrag
      end

      def ansatz_to_s
        ansatz
      end

    end

    class IstStunden < Eintrag
      def initialize(name, menge)
        super(name, menge, nil)
      end

      def betrag;
        0;
      end

      # überschreibt
      def betrag_gerundet_to_s
        nil
      end

    end

    class Zulage < Eintrag

      def initialize(name, menge, ansatz)
        super(name, menge, ansatz)
      end

      def betrag
        menge * ansatz
      end

      def ansatz_to_s
        value_gerundet_to_s(ansatz)
      end

    end

    class Abzug < Eintrag
      #ansatz = prozent
      def initialize name, menge, ansatz
        super(name, menge, ansatz)
      end

      def betrag
        menge * (ansatz / 100)
      end

      def menge_to_s
        menge.nil? ? nil : value_gerundet_to_s(menge)
      end

      def ansatz_to_s
        x = format('%.2f', ansatz)
        "#{x}%"
      end

    end

    class Rechnung < Eintrag

      def initialize name, positive, negative, einheit = "Fr"
        super(name, nil, nil, einheit)
        @positive, @negative = positive, negative
      end

      def betrag
        v = 0
        v = @positive.inject(v) { |v, x| v + x.betrag }
        v = @negative.inject(v) { |v, x| v - x.betrag }
        v
      end
    end
  end
end